@@ -43,6 +43,7 @@ import re
 import sys
 import ssl
 import warnings
+import getpass
 
 try:
     from http.cookiejar import LWPCookieJar, CookieJar
@@ -63,7 +64,9 @@ except ImportError:
 
 from . import OscConfigParser
 from osc import oscerr
+from osc.util.helper import raw_input
 from .oscsslexcp import NoSecureSSLError
+from osc import credentials
 
 GENERIC_KEYRING = False
 GNOME_KEYRING = False
@@ -76,12 +79,7 @@ except:
         import gobject
         gobject.set_application_name('osc')
         import gnomekeyring
-        if os.environ['GNOME_DESKTOP_SESSION_ID']:
-            # otherwise gnome keyring bindings spit out errors, when you have
-            # it installed, but you are not under gnome
-            # (even though hundreds of gnome-keyring daemons got started in parallel)
-            # another option would be to support kwallet here
-            GNOME_KEYRING = gnomekeyring.is_available()
+        GNOME_KEYRING = gnomekeyring.is_available()
     except:
         pass
 
@@ -97,9 +95,9 @@ def _get_processors():
         return 1
 
 DEFAULTS = {'apiurl': 'https://api.opensuse.org',
-            'user': 'your_username',
-            'pass': 'your_password',
-            'passx': '',
+            'user': None,
+            'pass': None,
+            'passx': None,
             'packagecachedir': '/var/tmp/osbuild-packagecache',
             'su-wrapper': 'sudo',
 
@@ -151,8 +149,8 @@ DEFAULTS = {'apiurl': 'https://api.opensuse.org',
             'checkout_rooted': '0',
             # local files to ignore with status, addremove, ....
             'exclude_glob': '.osc CVS .svn .* _linkerror *~ #*# *.orig *.bak *.changes.vctmp.*',
-            # whether to keep passwords in plaintext.
-            'plaintext_passwd': '1',
+            # whether to keep passwords in plaintext (deprecated (see creds manager)).
+            'plaintext_passwd': '0',
             # limit the age of requests shown with 'osc req list'.
             # this is a default only, can be overridden by 'osc req list -D NNN'
             # Use 0 for unlimted.
@@ -160,7 +158,7 @@ DEFAULTS = {'apiurl': 'https://api.opensuse.org',
             # check for unversioned/removed files before commit
             'check_filelist': '1',
             # check for pending requests after executing an action (e.g. checkout, update, commit)
-            'check_for_request_on_action': '0',
+            'check_for_request_on_action': '1',
             # what to do with the source package if the submitrequest has been accepted
             'submitrequest_on_accept_action': '',
             'request_show_interactive': '0',
@@ -280,8 +278,11 @@ apiurl = %(apiurl)s
 # extra packages to install when building packages locally (osc build)
 # this corresponds to osc build's -x option and can be overridden with that
 # -x '' can also be given on the command line to override this setting, or
-# you can have an empty setting here.
-#extra-pkgs = vim gdb strace
+# you can have an empty setting here. This global setting may leads to 
+# dependency problems when the base distro is not providing the package.
+# => using server side definition via cli_debug_packages substitute rule is
+#    recommended therefore.
+#extra-pkgs = 
 
 # build platform is used if the platform argument is omitted to osc build
 #build_repository = %(build_repository)s
@@ -298,12 +299,6 @@ apiurl = %(apiurl)s
 # local files to ignore with status, addremove, ....
 #exclude_glob = %(exclude_glob)s
 
-# keep passwords in plaintext.
-# Set to 0 to obfuscate passwords. It's no real security, just
-# prevents most people from remembering your password if they watch
-# you editing this file.
-#plaintext_passwd = %(plaintext_passwd)s
-
 # limit the age of requests shown with 'osc req list'.
 # this is a default only, can be overridden by 'osc req list -D NNN'
 # Use 0 for unlimted.
@@ -327,14 +322,11 @@ apiurl = %(apiurl)s
 # print call traces in case of errors
 #traceback = 1
 
-# use KDE/Gnome/MacOS/Windows keyring for credentials if available
-#use_keyring = 1
-
 # check for unversioned/removed files before commit
 #check_filelist = 1
 
 # check for pending requests after executing an action (e.g. checkout, update, commit)
-#check_for_request_on_action = 0
+#check_for_request_on_action = 1
 
 # what to do with the source package if the submitrequest has been accepted. If
 # nothing is specified the API default is used
@@ -358,8 +350,6 @@ apiurl = %(apiurl)s
 #review_inherit_group = 1
 
 [%(apiurl)s]
-user = %(user)s
-pass = %(pass)s
 # set aliases for this apiurl
 # aliases = foo, bar
 # real name used in .changes, unless the one from osc meta prj <user> will be used
@@ -371,8 +361,6 @@ pass = %(pass)s
 #       User: mumblegack
 # Plain text password
 #pass =
-# Force using of keyring for this API
-#keyring = 1
 """
 
 
@@ -672,7 +660,7 @@ def write_config(fname, cp):
         raise
 
 
-def config_set_option(section, opt, val=None, delete=False, update=True, **kwargs):
+def config_set_option(section, opt, val=None, delete=False, update=True, creds_mgr_descr=None, **kwargs):
     """
     Sets a config option. If val is not specified the current/default value is
     returned. If val is specified, opt is set to val and the new value is returned.
@@ -708,11 +696,37 @@ def config_set_option(section, opt, val=None, delete=False, update=True, **kwarg
         raise oscerr.ConfigError('unknown config option \'%s\'' % opt, config['conffile'])
     run = False
     if val:
-        cp.set(section, opt, val)
-        write_config(config['conffile'], cp)
+        if opt == 'pass':
+            creds_mgr = _get_credentials_manager(section, cp)
+            user = _extract_user_compat(cp, section, creds_mgr)
+            old_pw = creds_mgr.get_password(section, user, defer=False)
+            try:
+                creds_mgr.delete_password(section, user)
+                if creds_mgr_descr:
+                    creds_mgr_new = creds_mgr_descr.create(cp)
+                else:
+                    creds_mgr_new = creds_mgr
+                creds_mgr_new.set_password(section, user, val)
+                write_config(config['conffile'], cp)
+                opt = credentials.AbstractCredentialsManager.config_entry
+                old_pw = None
+            finally:
+                if old_pw is not None:
+                    creds_mgr.set_password(section, user, old_pw)
+                    # not nice, but needed if the Credentials Manager will change
+                    # something in cp
+                    write_config(config['conffile'], cp)
+        else:
+            cp.set(section, opt, val)
+            write_config(config['conffile'], cp)
         run = True
-    elif delete and cp.has_option(section, opt):
-        cp.remove_option(section, opt)
+    elif delete and (cp.has_option(section, opt) or opt == 'pass'):
+        if opt == 'pass':
+            creds_mgr = _get_credentials_manager(section, cp)
+            user = _extract_user_compar(cp, section, creds_mgr)
+            creds_mgr.delete_password(section, user)
+        else:
+            cp.remove_option(section, opt)
         write_config(config['conffile'], cp)
         run = True
     if run and update:
@@ -725,15 +739,17 @@ def config_set_option(section, opt, val=None, delete=False, update=True, **kwarg
         return (opt, cp.get(section, opt, raw=True))
     return (opt, None)
 
-def passx_decode(passx):
-    """decode the obfuscated password back to plain text password"""
-    return bz2.decompress(base64.b64decode(passx.encode("ascii"))).decode("ascii")
+def _extract_user_compat(cp, section, creds_mgr):
+    """
+    This extracts the user either from the ConfigParser or
+    the creds_mgr. Only needed for deprecated Gnome Keyring
+    """
+    user = cp.get(section, 'user')
+    if user is None and hasattr(creds_mgr, 'get_user'):
+        user = creds_mgr.get_user(section)
+    return user
 
-def passx_encode(passwd):
-    """encode plain text password to obfuscated form"""
-    return base64.b64encode(bz2.compress(passwd.encode('ascii'))).decode("ascii")
-
-def write_initial_config(conffile, entries, custom_template=''):
+def write_initial_config(conffile, entries, custom_template='', creds_mgr_descriptor=None):
     """
     write osc's intial configuration file. entries is a dict which contains values
     for the config file (e.g. { 'user' : 'username', 'pass' : 'password' } ).
@@ -742,37 +758,19 @@ def write_initial_config(conffile, entries, custom_template=''):
     conf_template = custom_template or new_conf_template
     config = DEFAULTS.copy()
     config.update(entries)
-    # at this point use_keyring and gnome_keyring are str objects
-    if config['use_keyring'] == '1' and GENERIC_KEYRING:
-        protocol, host, path = \
-            parse_apisrv_url(None, config['apiurl'])
-        keyring.set_password(host, config['user'], config['pass'])
-        config['pass'] = ''
-        config['passx'] = ''
-    elif config['gnome_keyring'] == '1' and GNOME_KEYRING:
-        protocol, host, path = \
-            parse_apisrv_url(None, config['apiurl'])
-        gnomekeyring.set_network_password_sync(
-            user=config['user'],
-            password=config['pass'],
-            protocol=protocol,
-            server=host,
-            object=path)
-        config['user'] = ''
-        config['pass'] = ''
-        config['passx'] = ''
-    if not config['plaintext_passwd']:
-        config['pass'] = ''
-    else:
-        config['passx'] = passx_encode(config['pass'])
-
     sio = StringIO(conf_template.strip() % config)
     cp = OscConfigParser.OscConfigParser(DEFAULTS)
     cp.readfp(sio)
+    cp.set(config['apiurl'], 'user', config['user'])
+    if creds_mgr_descriptor:
+        creds_mgr = creds_mgr_descriptor.create(cp)
+    else:
+        creds_mgr = _get_credentials_manager(config['apiurl'], cp)
+    creds_mgr.set_password(config['apiurl'], config['user'], config['pass'])
     write_config(conffile, cp)
 
 
-def add_section(filename, url, user, passwd):
+def add_section(filename, url, user, passwd, creds_mgr_descriptor=None):
     """
     Add a section to config file for new api url.
     """
@@ -783,33 +781,39 @@ def add_section(filename, url, user, passwd):
     except OscConfigParser.configparser.DuplicateSectionError:
         # Section might have existed, but was empty
         pass
+    cp.set(url, 'user', user)
+    if creds_mgr_descriptor:
+        creds_mgr = creds_mgr_descriptor.create(cp)
+    else:
+        creds_mgr = _get_credentials_manager(url, cp)
+    creds_mgr.set_password(url, user, passwd)
+    write_config(filename, cp)
+
+
+def _get_credentials_manager(url, cp):
+    if cp.has_option(url, credentials.AbstractCredentialsManager.config_entry):
+        creds_mgr = credentials.create_credentials_manager(url, cp)
+        if creds_mgr is None:
+            msg = 'Unable to instantiate creds mgr (section: %s)' % url
+            conffile = get_configParser.conffile
+            raise oscerr.ConfigMissingCredentialsError(msg, conffile, url)
+        return creds_mgr
     if config['use_keyring'] and GENERIC_KEYRING:
-        protocol, host, path = parse_apisrv_url(None, url)
-        keyring.set_password(host, user, passwd)
-        cp.set(url, 'keyring', '1')
-        cp.set(url, 'user', user)
-        cp.remove_option(url, 'pass')
-        cp.remove_option(url, 'passx')
+        return credentials.get_keyring_credentials_manager(cp)
     elif config['gnome_keyring'] and GNOME_KEYRING:
         protocol, host, path = parse_apisrv_url(None, url)
-        gnomekeyring.set_network_password_sync(
-            user=user,
-            password=passwd,
-            protocol=protocol,
-            server=host,
-            object=path)
-        cp.set(url, 'keyring', '1')
-        cp.remove_option(url, 'pass')
-        cp.remove_option(url, 'passx')
-    else:
-        cp.set(url, 'user', user)
-        if not config['plaintext_passwd']:
-            cp.remove_option(url, 'pass')
-            cp.set(url, 'passx', passx_encode(passwd))
-        else:
-            cp.remove_option(url, 'passx')
-            cp.set(url, 'pass', passwd)
-    write_config(filename, cp)
+        return credentials.GnomeKeyringCredentialsManager(cp, None)
+    elif cp.get(url, 'passx') is not None:
+        return credentials.ObfuscatedConfigFileCredentialsManager(cp, None)
+    return credentials.PlaintextConfigFileCredentialsManager(cp, None)
+
+
+class APIHostOptionsEntry(dict):
+    def __getitem__(self, key, *args, **kwargs):
+        value = super(self.__class__, self).__getitem__(key, *args, **kwargs)
+        if key == 'pass' and callable(value):
+            value = value()
+        return value
 
 
 def get_config(override_conffile=None,
@@ -888,74 +892,16 @@ def get_config(override_conffile=None,
         # backward compatiblity
         scheme, host, path = parse_apisrv_url(config.get('scheme', 'https'), url)
         apiurl = urljoin(scheme, host, path)
-        user = None
-        password = None
-        if config['use_keyring'] and GENERIC_KEYRING:
-            try:
-                # Read from keyring lib if available
-                user = cp.get(url, 'user', raw=True)
-                password = str(keyring.get_password(host, user))
-            except:
-                # Fallback to file based auth.
-                pass
-        elif config['gnome_keyring'] and GNOME_KEYRING:
-            # Read from gnome keyring if available
-            try:
-                gk_data = gnomekeyring.find_network_password_sync(protocol=scheme, server=host, object=path)
-                if not 'user' in gk_data[0]:
-                    raise oscerr.ConfigError('no user found in keyring', conffile)
-                user = gk_data[0]['user']
-                if 'password' in gk_data[0]:
-                    password = str(gk_data[0]['password'])
-                else:
-                    # this is most likely an error
-                    print('warning: no password found in keyring', file=sys.stderr)
-            except gnomekeyring.NoMatchError:
-                # Fallback to file based auth.
-                pass
-
-        if not user is None and len(user) == 0:
-            user = None
-            print('Warning: blank user in the keyring for the ' \
-                'apiurl %s.\nPlease fix your keyring entry.', file=sys.stderr)
-
-        if user is not None and password is None:
-            err = ('no password defined for "%s".\nPlease fix your keyring '
-                   'entry or gnome-keyring setup.\nAssuming an empty password.'
-                   % url)
-            print(err, file=sys.stderr)
-            password = ''
-
-        # Read credentials from config
+        creds_mgr = _get_credentials_manager(url, cp)
+        # if the deprecated gnomekeyring is used we should use the apiurl instead of url
+        # (that's what the old code did), but this makes things more complex
+        # (also, it is very unlikely that url and apiurl differ)
+        user = _extract_user_compat(cp, url, creds_mgr)
         if user is None:
-            #FIXME: this could actually be the ideal spot to take defaults
-            #from the general section.
-            user = cp.get(url, 'user', raw=True)        # need to set raw to prevent '%' expansion
-            password = cp.get(url, 'pass', raw=True)    # especially on password!
-            try:
-                passwordx = passx_decode(cp.get(url, 'passx', raw=True))  # especially on password!
-            except:
-                passwordx = ''
-
-            if password == None or password == 'your_password':
-                password = ''
-
-            if user is None or user == '':
-                raise oscerr.ConfigError('user is blank for %s, please delete or complete the "user=" entry in %s.' % (apiurl, config['conffile']), config['conffile'])
-
-            if config['plaintext_passwd'] and passwordx or not config['plaintext_passwd'] and password:
-                if config['plaintext_passwd']:
-                    if password != passwordx:
-                        print('%s: rewriting from encoded pass to plain pass' % url, file=sys.stderr)
-                    add_section(conffile, url, user, passwordx)
-                    password = passwordx
-                else:
-                    if password != passwordx:
-                        print('%s: rewriting from plain pass to encoded pass' % url, file=sys.stderr)
-                    add_section(conffile, url, user, password)
-
-            if not config['plaintext_passwd']:
-                password = passwordx
+            raise oscerr.ConfigMissingCredentialsError('No user found in section %s' % url, conffile, url)
+        password = creds_mgr.get_password(url, user)
+        if password is None:
+            raise oscerr.ConfigMissingCredentialsError('No password found in section %s' % url, conffile, url)
 
         if cp.has_option(url, 'http_headers'):
             http_headers = cp.get(url, 'http_headers')
@@ -972,9 +918,10 @@ def get_config(override_conffile=None,
                     raise oscerr.ConfigError(msg, conffile)
                 aliases[key] = url
 
-        api_host_options[apiurl] = {'user': user,
-                                    'pass': password,
-                                    'http_headers': http_headers}
+        entry = {'user': user,
+                 'pass': password,
+                 'http_headers': http_headers}
+        api_host_options[apiurl] = APIHostOptionsEntry(entry)
 
         optional = ('realname', 'email', 'sslcertck', 'cafile', 'capath')
         for key in optional:
@@ -1015,6 +962,8 @@ def get_config(override_conffile=None,
     if 'build_platform' in config:
         print('Warning: Use of \'build_platform\' config option is deprecated! (use \'build_repository\' instead)', file=sys.stderr)
         config['build_repository'] = config['build_platform']
+    if config['plaintext_passwd']:
+        print('The \'plaintext_passwd\' option is deprecated and will be ignored', file=sys.stderr)
 
     config['verbose'] = int(config['verbose'])
     # override values which we were called with
@@ -1061,4 +1010,30 @@ def identify_conf():
 
     return conffile
 
+def interactive_config_setup(conffile, apiurl, initial=True):
+    user = raw_input('Username: ')
+    passwd = getpass.getpass()
+    creds_mgr_descr = select_credentials_manager_descr()
+    if initial:
+        config = {'user': user, 'pass': passwd}
+        if apiurl:
+            config['apiurl'] = apiurl
+        write_initial_config(conffile, config, creds_mgr_descriptor=creds_mgr_descr)
+    else:
+        add_section(conffile, apiurl, user, passwd, creds_mgr_descriptor=creds_mgr_descr)
+
+def select_credentials_manager_descr():
+    if not credentials.has_keyring_support():
+        print('To use keyrings please install python-keyring.')
+    creds_mgr_descriptors = credentials.get_credentials_manager_descriptors()
+    for i, creds_mgr_descr in enumerate(creds_mgr_descriptors, 1):
+        print('%d) %s (%s)' % (i, creds_mgr_descr.name(), creds_mgr_descr.description()))#
+    i = raw_input('Select credentials manager: ')
+    if not i.isdigit():
+        sys.exit('Invalid selection')
+    i = int(i) - 1
+    if i < 0 or i >= len(creds_mgr_descriptors):
+        sys.exit('Invalid selection')
+    return creds_mgr_descriptors[i]
+
 # vim: sw=4 et
