@@ -28,7 +28,8 @@ from .meter import create_text_meter
 
 class Fetcher:
     def __init__(self, cachedir='/tmp', api_host_options={}, urllist=[],
-            http_debug=False, cookiejar=None, offline=False, enable_cpio=True):
+                 http_debug=False, cookiejar=None, offline=False,
+                 enable_cpio=True, modules=[]):
         # set up progress bar callback
         self.progress_obj = None
         if sys.stdout.isatty():
@@ -36,6 +37,7 @@ class Fetcher:
 
         self.cachedir = cachedir
         self.urllist = urllist
+        self.modules = modules
         self.http_debug = http_debug
         self.offline = offline
         self.cpio = {}
@@ -59,6 +61,8 @@ class Fetcher:
             return
         query = ['binary=%s' % quote_plus(i) for i in pkgs]
         query.append('view=cpio')
+        for module in self.modules:
+            query.append('module=' + module)
         try:
             url = makeurl(apiurl, ['build', project, repo, arch, package], query=query)
             sys.stdout.write("preparing download ...\r")
@@ -73,7 +77,7 @@ class Fetcher:
                     # getbinarylist instead of the public/... route
                     # (which is routed to getbinaries)
                     # getbinaries does not support kiwi builds
-                    if hdr.filename == '.errors':
+                    if hdr.filename == b'.errors':
                         archive.copyin_file(hdr.filename)
                         raise oscerr.APIError('CPIO archive is incomplete '
                                               '(see .errors file)')
@@ -286,7 +290,7 @@ class Fetcher:
 
             if try_parent:
                 if self.http_debug:
-                    print("can't fetch key for %s: %s" % (i, e.strerror), file=sys.stderr)
+                    print("can't fetch key for %s" % (i), file=sys.stderr)
                     print("url: %s" % url, file=sys.stderr)
 
                 if os.path.exists(dest):
