@@ -341,6 +341,7 @@ class TestRequest(OscTestCase):
         self.assertEqual(r.state.name, 'accepted')
         self.assertEqual(r.state.when, '2010-12-27T01:36:29')
         self.assertEqual(r.state.who, 'user1')
+        self.assertEqual(r.state.approver, None)
         self.assertEqual(r.state.comment, '')
         self.assertEqual(r.statehistory[0].when, '2010-12-13T13:02:03')
         self.assertEqual(r.statehistory[0].who, 'creator')
@@ -373,6 +374,7 @@ class TestRequest(OscTestCase):
         self.assertTrue(r.actions[1].tgt_package is None)
         self.assertEqual(r.state.name, 'review')
         self.assertEqual(r.state.when, '2010-12-27T01:36:29')
+        self.assertEqual(r.state.approver, 'someone')
         self.assertEqual(r.state.who, 'abc')
         self.assertEqual(r.state.comment, '')
         self.assertEqual(r.reviews[0].state, 'new')
